@@ -1,36 +1,52 @@
 import asyncio
 import logging
 import re
-from typing import TYPE_CHECKING, Final, Tuple
+from pathlib import Path
+from typing import AsyncIterator, Final
 
 from aiotgbot import (Bot, BotUpdate, ContentType, GroupChatFilter,
-                      HandlerTable, ParseMode, PrivateChatFilter,
-                      TelegramError)
-from aiotgbot.api_types import BotCommand
+                      HandlerTable, ParseMode, PollBot, PrivateChatFilter,
+                      Runner, TelegramError)
+from aiotgbot.api_types import (BotCommand, BotCommandScopeAllPrivateChats,
+                                BotCommandScopeChat)
 from aiotgbot.storage_sqlite import SQLiteStorage
 
-from .helpers import (ADMIN_USERNAME_KEY, CHAT_LIST_KEY, CHAT_LIST_SIZE_KEY,
-                      AlbumForwarder, FromAdminFilter, FromUserFilter, Stopped,
-                      add_chat_to_list, chat_key, get_chat, get_software, path,
-                      remove_chat_from_list, reply_menu, send_from_message,
-                      send_user_message, set_chat, user_link)
-
-logger = logging.getLogger('feedback_bot')
+from .helpers import (CHAT_LIST_KEY, CONFIG_KEY, REPLY_PREFIX, AlbumForwarder,
+                      Config, FromAdminFilter, FromUserFilter, Stopped,
+                      add_chat_to_list, chat_key, debug, get_chat,
+                      get_software, path, remove_chat_from_list, reply_menu,
+                      send_from_message, send_user_message, set_chat,
+                      user_link)
 
 SOFTWARE: Final[str] = get_software()
-COMMANDS: Final[Tuple[BotCommand, ...]] = (
+USER_COMMANDS: Final[tuple[BotCommand, ...]] = (
     BotCommand('start', 'Начать работу'),
     BotCommand('help', 'Помощь'),
     BotCommand('stop', 'Остановить')
 )
-REPLY_RXP: Final[re.Pattern] = re.compile(r'^reply-to-(?P<chat_id>\d+)$')
+ADMIN_COMMANDS: Final[tuple[BotCommand, ...]] = (
+    BotCommand('help', 'Помощь'),
+    BotCommand('reply', 'Ответить пользователю'),
+    BotCommand('add_to_group', 'Добавить в группу'),
+    BotCommand('remove_from_group', 'Удалить из группы'),
+    BotCommand('reset', 'Сбросить состояние')
+)
+GROUP_COMMANDS: Final[tuple[BotCommand, ...]] = (
+    BotCommand('help', 'Помощь'),
+    BotCommand('reply', 'Ответить пользователю')
+)
+CHAT_ID_GROUP: Final[str] = 'chat_id'
+REPLY_RXP: 'Final[re.Pattern[str]]' = re.compile(
+    rf'^{REPLY_PREFIX}\|(?P<{CHAT_ID_GROUP}>-?\d+)$')
 ALBUM_FORWARDER_KEY: Final[str] = 'album_forwarder'
 GROUP_CHAT_KEY: Final[str] = 'group_chat'
 ADMIN_CHAT_ID_KEY: Final[str] = 'admin_chat_id'
 CURRENT_CHAT_KEY: Final[str] = 'current_chat'
 WAIT_REPLY_FROM_ID_KEY: Final[str] = 'wait_reply_from_id'
+TZ_KEY: Final['str'] = 'TZ'
 
 
+logger = logging.getLogger('feedback-bot')
 handlers = HandlerTable()
 
 
@@ -40,14 +56,16 @@ async def user_start_command(bot: Bot, update: BotUpdate) -> None:
     assert update.message is not None
     assert update.message.from_ is not None
     logger.info('Start command from "%s"', update.message.from_.to_dict())
-    await Stopped.delete(bot, update.message.from_.id)
-    await set_chat(bot.storage, chat_key(update.message.chat.id),
-                   update.message.chat)
+    stopped = await Stopped.get(bot, update.message.chat.id)
+    if stopped is not None:
+        await Stopped.delete(bot, update.message.chat.id)
+        await bot.send_message(update.message.chat.id, 'С возвращением!')
+    await set_chat(bot, chat_key(update.message.chat.id), update.message.chat)
     await bot.send_message(update.message.chat.id,
                            'Пришлите сообщение или задайте вопрос. '
                            'Также вы можете использовать следующие команды:\n'
-                           '/help - помощь\n'
-                           '/stop - остановить и не получать больше сообщения')
+                           '/help — Помощь\n'
+                           '/stop — Остановить и не получать больше сообщения')
 
 
 @handlers.message(commands=['help'],
@@ -59,8 +77,8 @@ async def user_help_command(bot: Bot, update: BotUpdate) -> None:
     await bot.send_message(update.message.chat.id,
                            'Пришлите сообщение или задайте вопрос. '
                            'Также вы можете использовать следующие команды:\n'
-                           '/help - помощь\n'
-                           '/stop - остановить и не получать больше сообщения')
+                           '/help — Помощь\n'
+                           '/stop — Остановить и не получать больше сообщения')
 
 
 @handlers.message(commands=['stop'],
@@ -72,21 +90,21 @@ async def user_stop_command(bot: Bot, update: BotUpdate) -> None:
     stopped = Stopped()
     await stopped.set(bot, update.message.from_.id)
     await remove_chat_from_list(bot, update.message.from_.id)
-    group_chat = await get_chat(bot.storage, GROUP_CHAT_KEY)
-    if group_chat is not None:
-        notify_chat = group_chat
+    group_chat = await get_chat(bot, GROUP_CHAT_KEY)
+    if group_chat is None:
+        notify_chat_id = await bot.storage.get(ADMIN_CHAT_ID_KEY)
+        assert isinstance(notify_chat_id, int)
     else:
-        notify_chat = await bot.storage.get(ADMIN_CHAT_ID_KEY)
-    assert notify_chat is not None
+        notify_chat_id = group_chat.id
     await bot.send_message(
-        notify_chat.id,
+        notify_chat_id,
         f'{user_link(update.message.from_)} меня заблокировал '
         f'{stopped.dt:%Y-%m-%d %H:%M:%S %Z}.',
         parse_mode=ParseMode.HTML)
-    current_chat = await get_chat(bot.storage, CURRENT_CHAT_KEY)
+    current_chat = await get_chat(bot, CURRENT_CHAT_KEY)
     if current_chat is not None and current_chat.id == update.message.from_.id:
         await bot.storage.set(WAIT_REPLY_FROM_ID_KEY)
-        await set_chat(bot.storage, CURRENT_CHAT_KEY)
+        await set_chat(bot, CURRENT_CHAT_KEY)
 
 
 @handlers.message(commands=['start'],
@@ -97,11 +115,11 @@ async def admin_start_command(bot: Bot, update: BotUpdate) -> None:
     await bot.storage.set(ADMIN_CHAT_ID_KEY, update.message.chat.id)
 
     await bot.send_message(update.message.chat.id,
-                           '/help - помощь\n'
-                           '/reply - ответить пользователю\n'
-                           '/add_to_group - добавить в группу\n'
-                           '/remove_from_group - удалить из группы\n'
-                           '/reset - сбросить состояние')
+                           '/help — Помощь\n'
+                           '/reply — Ответить пользователю\n'
+                           '/add_to_group — Добавить в группу\n'
+                           '/remove_from_group — Удалить из группы\n'
+                           '/reset — Сбросить состояние')
 
 
 @handlers.message(commands=['help'],
@@ -110,11 +128,11 @@ async def admin_help_command(bot: Bot, update: BotUpdate) -> None:
     assert update.message is not None
     logger.info('Help command from admin')
     await bot.send_message(update.message.chat.id,
-                           '/help - помощь\n'
-                           '/reply - ответить пользователю\n'
-                           '/add_to_group - добавить в группу\n'
-                           '/remove_from_group - удалить из группы\n'
-                           '/reset - сбросить состояние')
+                           '/help — Помощь\n'
+                           '/reply — Ответить пользователю\n'
+                           '/add_to_group — Добавить в группу\n'
+                           '/remove_from_group — Удалить из группы\n'
+                           '/reset — Сбросить состояние')
 
 
 @handlers.message(commands=['reset'],
@@ -135,7 +153,7 @@ async def add_to_group_command(bot: Bot, update: BotUpdate) -> None:
 
     logger.info('Add to group command from "%s"',
                 update.message.from_.to_dict())
-    if await get_chat(bot.storage, GROUP_CHAT_KEY) is not None:
+    if await get_chat(bot, GROUP_CHAT_KEY) is not None:
         logger.info('Already in group. Ignore command')
         await bot.send_message(update.message.chat.id, 'Уже в группе.')
         return
@@ -155,7 +173,7 @@ async def remove_from_group_command(bot: Bot, update: BotUpdate) -> None:
     assert update.message.from_ is not None
     logger.info('Remove from group command from "%s"',
                 update.message.from_.to_dict())
-    group_chat = await get_chat(bot.storage, GROUP_CHAT_KEY)
+    group_chat = await get_chat(bot, GROUP_CHAT_KEY)
     if group_chat is None:
         logger.info('Not in group. Ignore command')
         await bot.send_message(update.message.chat.id, 'Не в группе.')
@@ -171,8 +189,8 @@ async def remove_from_group_command(bot: Bot, update: BotUpdate) -> None:
                            parse_mode=ParseMode.HTML,
                            disable_web_page_preview=True)
 
-    await set_chat(bot.storage, GROUP_CHAT_KEY)
-    await set_chat(bot.storage, CURRENT_CHAT_KEY)
+    await set_chat(bot, GROUP_CHAT_KEY)
+    await set_chat(bot, CURRENT_CHAT_KEY)
 
     logger.info('Removed from group "%s"', group_chat.to_dict())
 
@@ -184,15 +202,19 @@ async def group_start_command(bot: Bot, update: BotUpdate) -> None:
     assert update.message.from_ is not None
     logger.info('Start in group command from "%s"',
                 update.message.from_.to_dict())
-    if await get_chat(bot.storage, GROUP_CHAT_KEY):
+    if await get_chat(bot, GROUP_CHAT_KEY):
         logger.info('Attempt start in group "%s"',
                     update.message.chat.to_dict())
         return
 
-    await set_chat(bot.storage, GROUP_CHAT_KEY, update.message.chat)
-    await set_chat(bot.storage, CURRENT_CHAT_KEY)
+    await set_chat(bot, GROUP_CHAT_KEY, update.message.chat)
+    await set_chat(bot, CURRENT_CHAT_KEY)
+
+    await bot.set_my_commands(
+        GROUP_COMMANDS, BotCommandScopeChat(update.message.chat.id))
 
     admin_chat_id = await bot.storage.get(ADMIN_CHAT_ID_KEY)
+    assert isinstance(admin_chat_id, int)
     await bot.send_message(
         admin_chat_id, f'Запущен в <b>{update.message.chat.title}</b>.',
         parse_mode=ParseMode.HTML, disable_web_page_preview=True)
@@ -207,8 +229,8 @@ async def group_help_command(bot: Bot, update: BotUpdate) -> None:
     logger.info('Help message in group from "%s"',
                 update.message.from_.to_dict())
     await bot.send_message(update.message.chat.id,
-                           '/help - помощь\n'
-                           '/reply - ответить пользователю')
+                           '/help — Помощь\n'
+                           '/reply — Ответить пользователю')
 
 
 @handlers.message(commands=['reply'],
@@ -218,7 +240,7 @@ async def admin_reply_command(bot: Bot, update: BotUpdate) -> None:
     assert update.message.from_ is not None
     logger.info('Reply command from admin "%s"',
                 update.message.from_.to_dict())
-    group_chat = await get_chat(bot.storage, GROUP_CHAT_KEY)
+    group_chat = await get_chat(bot, GROUP_CHAT_KEY)
     if group_chat is not None:
         await bot.send_message(
             update.message.chat.id,
@@ -241,7 +263,7 @@ async def group_reply_command(bot: Bot, update: BotUpdate) -> None:
     assert update.message.from_ is not None
     logger.info('Reply in group command from "%s"',
                 update.message.from_.to_dict())
-    group_chat = await get_chat(bot.storage, GROUP_CHAT_KEY)
+    group_chat = await get_chat(bot, GROUP_CHAT_KEY)
     if group_chat is not None and group_chat.id != update.message.chat.id:
         await bot.leave_chat(update.message.chat.id)
         return
@@ -252,6 +274,7 @@ async def group_reply_command(bot: Bot, update: BotUpdate) -> None:
         return
     wait_reply_from_id = await bot.storage.get(WAIT_REPLY_FROM_ID_KEY)
     if wait_reply_from_id is not None:
+        assert isinstance(wait_reply_from_id, int)
         member = await bot.get_chat_member(update.message.chat.id,
                                            wait_reply_from_id)
         member_link = (user_link(member.user) if member.user.username is None
@@ -272,40 +295,50 @@ async def group_new_members(bot: Bot, update: BotUpdate) -> None:
     assert update.message is not None
     assert update.message.new_chat_members is not None
     logger.info('New group members message "%s"', update.message.chat)
-    group_chat = await get_chat(bot.storage, GROUP_CHAT_KEY)
+    me = await bot.get_me()
     for user in update.message.new_chat_members:
-        if user.username == (await bot.get_me()).username:
+        if user.id == me.id:
+            admin_chat_id = await bot.storage.get(ADMIN_CHAT_ID_KEY)
+            assert isinstance(admin_chat_id, int)
             await bot.send_message(
-                await bot.storage.get(ADMIN_CHAT_ID_KEY),
+                admin_chat_id,
                 f'Добавлен в группу <b>{update.message.chat.title}</b>.',
                 parse_mode=ParseMode.HTML, disable_web_page_preview=True)
             logger.info('Bot added to grouip "%s"',
                         update.message.chat.to_dict())
-
+            group_chat = await get_chat(bot, GROUP_CHAT_KEY)
             if (
                 group_chat is not None and
                 group_chat.id != update.message.chat.id
             ):
                 await bot.leave_chat(update.message.chat.id)
-
+            elif (
+                group_chat is not None and
+                group_chat.id == update.message.chat.id
+            ):
+                await bot.set_my_commands(
+                    GROUP_COMMANDS,
+                    BotCommandScopeChat(update.message.chat.id))
             break
 
 
 @handlers.message(content_types=[ContentType.LEFT_CHAT_MEMBER])
-async def group_left_member(bot: Bot, update: BotUpdate):
+async def group_left_member(bot: Bot, update: BotUpdate) -> None:
     assert update.message is not None
     assert update.message.left_chat_member is not None
     logger.info('Left group member message "%s"', update.message.to_dict())
     me = await bot.get_me()
     if update.message.left_chat_member.id == me.id:
+        admin_chat_id = await bot.storage.get(ADMIN_CHAT_ID_KEY)
+        assert isinstance(admin_chat_id, int)
         await bot.send_message(
-            await bot.storage.get(ADMIN_CHAT_ID_KEY),
+            admin_chat_id,
             f'Вышел из группы <b>{update.message.chat.title}</b>.',
             parse_mode=ParseMode.HTML, disable_web_page_preview=True)
         logger.info('Leave chat "%s"', update.message.chat.title)
-        group_chat = await get_chat(bot.storage, GROUP_CHAT_KEY)
+        group_chat = await get_chat(bot, GROUP_CHAT_KEY)
         if group_chat is not None and update.message.chat.id == group_chat.id:
-            await set_chat(bot.storage, GROUP_CHAT_KEY)
+            await set_chat(bot, GROUP_CHAT_KEY)
             logger.info('Forget chat "%s"', update.message.chat.title)
 
 
@@ -316,18 +349,18 @@ async def user_message(bot: Bot, update: BotUpdate) -> None:
     assert update.message is not None
     assert update.message.from_ is not None
     logger.info('Message from "%s"', update.message.from_.to_dict())
-    await set_chat(bot.storage, chat_key(update.message.chat.id),
-                   update.message.chat)
+    await set_chat(bot, chat_key(update.message.chat.id), update.message.chat)
     stopped = await Stopped.get(bot, update.message.chat.id)
     if stopped is not None:
         await Stopped.delete(bot, update.message.chat.id)
         await bot.send_message(update.message.chat.id, 'С возвращением!')
 
-    group_chat = await get_chat(bot.storage, GROUP_CHAT_KEY)
-    if group_chat is not None:
-        forward_chat_id = group_chat.id
-    else:
+    group_chat = await get_chat(bot, GROUP_CHAT_KEY)
+    if group_chat is None:
         forward_chat_id = await bot.storage.get(ADMIN_CHAT_ID_KEY)
+        assert isinstance(forward_chat_id, int)
+    else:
+        forward_chat_id = group_chat.id
 
     if update.message.audio is not None or update.message.sticker is not None:
         logger.info('Message from user "%s" contains audio or sticker',
@@ -350,7 +383,7 @@ async def group_message(bot: Bot, update: BotUpdate) -> None:
     assert update.message.from_ is not None
     logger.info('Reply messgae in group from "%s"',
                 update.message.from_.to_dict())
-    group_chat = await get_chat(bot.storage, GROUP_CHAT_KEY)
+    group_chat = await get_chat(bot, GROUP_CHAT_KEY)
     if group_chat is not None and group_chat.id != update.message.chat.id:
         await bot.leave_chat(update.message.chat.id)
         return
@@ -366,14 +399,14 @@ async def group_message(bot: Bot, update: BotUpdate) -> None:
     await send_user_message(bot, update.message)
 
     await bot.storage.set(WAIT_REPLY_FROM_ID_KEY)
-    await set_chat(bot.storage, CURRENT_CHAT_KEY)
+    await set_chat(bot, CURRENT_CHAT_KEY)
 
 
 @handlers.message(filters=[PrivateChatFilter(), FromAdminFilter()])
 async def admin_message(bot: Bot, update: BotUpdate) -> None:
     assert update.message is not None
     logger.info('Message from admin "%s"', update.message.to_dict())
-    group_chat = await get_chat(bot.storage, GROUP_CHAT_KEY)
+    group_chat = await get_chat(bot, GROUP_CHAT_KEY)
     if group_chat is not None:
         await bot.send_message(
             update.message.chat.id,
@@ -389,7 +422,7 @@ async def admin_message(bot: Bot, update: BotUpdate) -> None:
     await send_user_message(bot, update.message)
 
     await bot.storage.set(WAIT_REPLY_FROM_ID_KEY)
-    await set_chat(bot.storage, CURRENT_CHAT_KEY)
+    await set_chat(bot, CURRENT_CHAT_KEY)
 
 
 @handlers.callback_query(data_match=REPLY_RXP)
@@ -403,8 +436,8 @@ async def reply_callback(bot: Bot, update: BotUpdate) -> None:
 
     data_match = REPLY_RXP.match(update.callback_query.data)
     assert data_match is not None, 'Reply to data not match format'
-    current_chat_id = int(data_match.group('chat_id'))
-    current_chat = await get_chat(bot.storage, chat_key(current_chat_id))
+    current_chat_id = int(data_match.group(CHAT_ID_GROUP))
+    current_chat = await get_chat(bot, chat_key(current_chat_id))
     if current_chat is None:
         await bot.edit_message_text(
             'Ошибка. Сообщение не отправить.',
@@ -424,7 +457,7 @@ async def reply_callback(bot: Bot, update: BotUpdate) -> None:
         return
     await bot.storage.set(WAIT_REPLY_FROM_ID_KEY,
                           update.callback_query.from_.id)
-    await set_chat(bot.storage, CURRENT_CHAT_KEY, current_chat)
+    await set_chat(bot, CURRENT_CHAT_KEY, current_chat)
     await bot.edit_message_text(
         f'Введите сообщение для {user_link(current_chat)}.',
         chat_id=update.callback_query.message.chat.id,
@@ -432,89 +465,95 @@ async def reply_callback(bot: Bot, update: BotUpdate) -> None:
         parse_mode=ParseMode.HTML, disable_web_page_preview=True)
 
 
-async def on_startup(bot: Bot) -> None:
-    if await bot.storage.get(CHAT_LIST_KEY) is None:
-        await bot.storage.set(CHAT_LIST_KEY, [])
-    if await bot.storage.get(CURRENT_CHAT_KEY) is None:
-        await bot.storage.set(CURRENT_CHAT_KEY)
-    if await bot.storage.get(ADMIN_CHAT_ID_KEY) is None:
-        await bot.storage.set(ADMIN_CHAT_ID_KEY)
-    if await bot.storage.get(GROUP_CHAT_KEY) is None:
-        await bot.storage.set(GROUP_CHAT_KEY)
+async def run_context(
+    _: Runner, storage_path: Path, config: Config
+) -> AsyncIterator[None]:
+    if debug():
+        asyncio.get_running_loop().slow_callback_duration = 0.01
+
+    storage = SQLiteStorage(storage_path)
+    await storage.connect()
+    if await storage.get(CHAT_LIST_KEY) is None:
+        await storage.set(CHAT_LIST_KEY, [])
+    if await storage.get(CURRENT_CHAT_KEY) is None:
+        await storage.set(CURRENT_CHAT_KEY)
+    if await storage.get(ADMIN_CHAT_ID_KEY) is None:
+        await storage.set(ADMIN_CHAT_ID_KEY)
+    if await storage.get(GROUP_CHAT_KEY) is None:
+        await storage.set(GROUP_CHAT_KEY)
+
+    handlers.freeze()
+    bot = PollBot(config.tg_token, handlers, storage)
+    bot[CONFIG_KEY] = config
+    await bot.start()
 
     album_forwarder = AlbumForwarder(bot)
-    await album_forwarder.start()
     bot[ALBUM_FORWARDER_KEY] = album_forwarder
+    await bot[ALBUM_FORWARDER_KEY].start()
 
-    if COMMANDS != await bot.get_my_commands():
-        logger.info('Update bot commands')
-        await bot.set_my_commands(COMMANDS)
+    admin_chat_id = await bot.storage.get(ADMIN_CHAT_ID_KEY)
+    group_chat = await get_chat(bot, GROUP_CHAT_KEY)
+    await bot.delete_my_commands()
+    await bot.set_my_commands(USER_COMMANDS, BotCommandScopeAllPrivateChats())
+    if admin_chat_id is not None:
+        assert isinstance(admin_chat_id, int)
+        await bot.set_my_commands(
+            ADMIN_COMMANDS, BotCommandScopeChat(admin_chat_id))
+    if group_chat is not None:
+        try:
+            await bot.set_my_commands(
+                GROUP_COMMANDS, BotCommandScopeChat(group_chat.id))
+        except TelegramError as exception:
+            if exception.error_code == 403:
+                logger.info('Can\'t set commands in chat "%s": %s',
+                            group_chat.id, exception.description)
+            else:
+                raise
 
+    yield
 
-async def on_shutdown(bot: Bot) -> None:
-    album_forwarder = bot.get(ALBUM_FORWARDER_KEY)
-    assert isinstance(album_forwarder, AlbumForwarder)
     await album_forwarder.stop()
+    await bot.stop()
+    await storage.close()
 
 
-def main():
-    import argparse
+def setup_logging() -> None:
     import os
 
-    parser = argparse.ArgumentParser(description='Feedback aiotgbot bot.')
-    parser.add_argument('storage_path', type=path,
-                        help='aiotgbot bot API token')
-    parser.add_argument('-a', dest='admin_username',
-                        default=os.environ.get('ADMIN_USERNAME', ''),
-                        type=str, help='admin username')
-    parser.add_argument('-t', dest='token',
-                        default=os.environ.get('TG_BOT_TOKEN', ''),
-                        type=str, help='aiotgbot bot API token')
-    parser.add_argument('-l', dest='chat_list_size', type=int,
-                        default=os.environ.get('CHAT_LIST_SIZE', 10),
-                        help='size of chat list')
-    parser.add_argument('-d', dest='debug', action='store_true',
-                        help='enable debug mode')
-    args = parser.parse_args()
-
-    if args.admin_username == '':
-        parser.error('admin username is empty')
-
-    if args.token == '':
-        parser.error('token is empty')
-
-    debug = args.debug or os.environ.get('DEBUG', '0') == '1'
-
-    if not (args.storage_path.is_file() or args.storage_path.parent.is_dir()):
-        parser.error(f'config file "{args.storage_path}" does not exist '
-                     f'and parent path is not dir')
-
     log_format = '%(asctime)s %(name)s %(levelname)s: %(message)s'
-    if debug:
+    if debug():
         logging.basicConfig(level=logging.DEBUG, format=log_format)
         logging.getLogger('asyncio').setLevel(logging.ERROR)
         logging.getLogger('aiosqlite').setLevel(logging.INFO)
-        logger.debug('PYTHONOPTIMIZE=%s', os.environ.get('PYTHONOPTIMIZE'))
     else:
         logging.basicConfig(level=logging.INFO, format=log_format)
+    logger.info('PYTHONOPTIMIZE=%s', os.environ.get('PYTHONOPTIMIZE'))
     logger.info(SOFTWARE)
 
-    storage = SQLiteStorage(args.storage_path)
-    feedback_bot = Bot(args.token, handlers, storage)
-    feedback_bot[ADMIN_USERNAME_KEY] = args.admin_username
-    feedback_bot[CHAT_LIST_SIZE_KEY] = args.chat_list_size
 
-    if not TYPE_CHECKING:
-        try:
-            import uvloop  # noqa
-        except ImportError:
-            pass
-        else:
-            asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
+def main() -> None:
+    import argparse  # isort:skip
+    import os  # isort:skip
+    import uvloop
 
-    asyncio.run(feedback_bot.poll(on_startup=on_startup,
-                                  on_shutdown=on_shutdown),
-                debug=debug)
+    parser = argparse.ArgumentParser(description='Feedback aiotgbot bot')
+    parser.add_argument('config_path', type=path, help='config path')
+    parser.add_argument('storage_path', type=path, help='storage path')
+    args = parser.parse_args()
+    if not args.config_path.exists():
+        parser.error(f'config file "{args.config_path}" does not exist')
+    if not (args.storage_path.is_file() or args.storage_path.parent.is_dir()):
+        parser.error(f'config file "{args.storage_path}" does not exist '
+                     f'and parent path is not dir')
+    if TZ_KEY not in os.environ:
+        parser.error('Env var TZ is not set')
+    config = Config.load(args.config_path)
+
+    setup_logging()
+    uvloop.install()
+    runner = Runner(run_context, debug=debug(), storage_path=args.storage_path,
+                    config=config)
+    runner.run()
 
 
 if __name__ == '__main__':  # pragma: nocover
